@@ -32,7 +32,7 @@
 #' @format A data frame with 79 rows and 4 variables:
 #' \describe{
 #'
-#' \item{NV}{neovasculation with coding 0 for absent and 1 for present}
+#' \item{NV}{neovasculization with coding 0 for absent and 1 for present}
 #'
 #' \item{PI}{pulsality index of arteria uterina}
 #'
@@ -109,3 +109,37 @@
 #'
 "alligators"
 
+
+
+#' Opinion on Stem Cell Research and Religious Fundamentalism
+#'
+#' A data set from the 2006 General Social Survey that shows the
+#' relationship in the United States between opinion about funding
+#' stem cell research and the fundamentalism/liberalism of one’s
+#' religious beliefs, stratified by gender.
+#'
+#' @format A data frame with 24 rows and 4 variables:
+#' \describe{
+#'
+#' \item{research}{opinion about funding stem cell research with levels \code{definitely}, \code{probably}, \code{probably not}, \code{definitely not}}
+#'
+#' \item{gender}{the gender of the respondent with levels \code{female} and \code{male}}
+#'
+#' \item{religion}{the fundamentalism/liberalism of one’s religious
+#' beliefs with levels \code{fundamentalist}, \code{moderate},
+#' \code{liberal}}
+#'
+#' \item{frequency}{the number of times a respondent fell in each of the combinations of levels for \code{research}, \code{religion} and \code{gender}}
+#'
+#' }
+#'
+#' @source
+#'
+#' The \code{stemcell} data set is analysed in Agresti (2010, Subsection~4.1.5).
+#'
+#' @references
+#'
+#' Agresti, A. (2010). *Analysis of Ordinal Categorical Data* (2nd edition).  Wiley Series in
+#' Probability and Statistics. Wiley
+#'
+"stemcell"
